/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_All.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/SingleValueIterator.h"

extern QoreClass* QC_SINGLEVALUEITERATOR;
/* Qore class Qore::<value> */

qore_classid_t CID_PSEUDOVALUE;
QoreClass* QC_PSEUDOVALUE;

// bool PseudoValue::callp(){}
static QoreValue PseudoValue_callp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 282 "Pseudo_QC_All.qpp"
    return false;
}

// bool PseudoValue::complexType(){}
static QoreValue PseudoValue_complexType(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 114 "Pseudo_QC_All.qpp"
    return false;
}

// bool PseudoValue::empty(){}
static QoreValue PseudoValue_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 165 "Pseudo_QC_All.qpp"
    return true;
}

// string PseudoValue::fullType(*bool with_namespaces){}
static QoreValue PseudoValue_fullType_Nb(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool with_namespaces = HARD_QORE_VALUE_BOOL(args, 0);
# 146 "Pseudo_QC_All.qpp"
    return new QoreStringNode(v.getFullTypeName(with_namespaces));
}

// bool PseudoValue::intp(){}
static QoreValue PseudoValue_intp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 254 "Pseudo_QC_All.qpp"
    return false;
}

// AbstractIterator PseudoValue::iterator(){}
static QoreValue PseudoValue_iterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 370 "Pseudo_QC_All.qpp"
    return new QoreObject(QC_SINGLEVALUEITERATOR, 0, new SingleValueIterator(v));
}

// int PseudoValue::lsize(){}
static QoreValue PseudoValue_lsize(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 298 "Pseudo_QC_All.qpp"
    return 1;
}

// int PseudoValue::size(){}
static QoreValue PseudoValue_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 186 "Pseudo_QC_All.qpp"
    return 0;
}

// bool PseudoValue::sizep(){}
static QoreValue PseudoValue_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 381 "Pseudo_QC_All.qpp"
    return false;
}

// bool PseudoValue::strp(){}
static QoreValue PseudoValue_strp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 268 "Pseudo_QC_All.qpp"
    return false;
}

// bool PseudoValue::toBool(){}
static QoreValue PseudoValue_toBool(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 345 "Pseudo_QC_All.qpp"
    return v.getAsBool();
}

// float PseudoValue::toFloat(){}
static QoreValue PseudoValue_toFloat(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 334 "Pseudo_QC_All.qpp"
    return v.getAsFloat();
}

// int PseudoValue::toInt(){}
static QoreValue PseudoValue_toInt(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 323 "Pseudo_QC_All.qpp"
    return v.getAsBigInt();
}

// number PseudoValue::toNumber(){}
static QoreValue PseudoValue_toNumber(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 356 "Pseudo_QC_All.qpp"
    return QoreNumberNode::toNumber(v);
}

// string PseudoValue::toString(){}
static QoreValue PseudoValue_toString(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 311 "Pseudo_QC_All.qpp"
    QoreStringNodeValueHelper str(v);
   return str.getReferencedValue();
}

// string PseudoValue::type(){}
static QoreValue PseudoValue_type(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 89 "Pseudo_QC_All.qpp"
    return new QoreStringNode(v.getTypeName());
}

// int PseudoValue::typeCode(){}
static QoreValue PseudoValue_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 70 "Pseudo_QC_All.qpp"
    return v.getType();
}

// bool PseudoValue::val(){}
static QoreValue PseudoValue_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 238 "Pseudo_QC_All.qpp"
    return false;
}

DLLLOCAL void preinitPseudoValueClass() {
    QC_PSEUDOVALUE = new QoreClass("<value>", "::Qore::<value>", QDOM_DEFAULT);
    CID_PSEUDOVALUE = QC_PSEUDOVALUE->getID();
    QC_PSEUDOVALUE->setSystem();
}

DLLLOCAL QoreClass* initPseudoValueClass() {
    if (!QC_PSEUDOVALUE)
        preinitPseudoValueClass();

    // bool PseudoValue::callp(){}
    QC_PSEUDOVALUE->addMethod("callp", (q_method_n_t)PseudoValue_callp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoValue::complexType(){}
    QC_PSEUDOVALUE->addMethod("complexType", (q_method_n_t)PseudoValue_complexType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoValue::empty(){}
    QC_PSEUDOVALUE->addMethod("empty", (q_method_n_t)PseudoValue_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoValue::fullType(*bool with_namespaces){}
    QC_PSEUDOVALUE->addMethod("fullType", (q_method_n_t)PseudoValue_fullType_Nb, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_namespaces");

    // bool PseudoValue::intp(){}
    QC_PSEUDOVALUE->addMethod("intp", (q_method_n_t)PseudoValue_intp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // AbstractIterator PseudoValue::iterator(){}
    QC_PSEUDOVALUE->addMethod("iterator", (q_method_n_t)PseudoValue_iterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

    // int PseudoValue::lsize(){}
    QC_PSEUDOVALUE->addMethod("lsize", (q_method_n_t)PseudoValue_lsize, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoValue::size(){}
    QC_PSEUDOVALUE->addMethod("size", (q_method_n_t)PseudoValue_size, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoValue::sizep(){}
    QC_PSEUDOVALUE->addMethod("sizep", (q_method_n_t)PseudoValue_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoValue::strp(){}
    QC_PSEUDOVALUE->addMethod("strp", (q_method_n_t)PseudoValue_strp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoValue::toBool(){}
    QC_PSEUDOVALUE->addMethod("toBool", (q_method_n_t)PseudoValue_toBool, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // float PseudoValue::toFloat(){}
    QC_PSEUDOVALUE->addMethod("toFloat", (q_method_n_t)PseudoValue_toFloat, Public, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo);

    // int PseudoValue::toInt(){}
    QC_PSEUDOVALUE->addMethod("toInt", (q_method_n_t)PseudoValue_toInt, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // number PseudoValue::toNumber(){}
    QC_PSEUDOVALUE->addMethod("toNumber", (q_method_n_t)PseudoValue_toNumber, Public, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo);

    // string PseudoValue::toString(){}
    QC_PSEUDOVALUE->addMethod("toString", (q_method_n_t)PseudoValue_toString, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoValue::type(){}
    QC_PSEUDOVALUE->addMethod("type", (q_method_n_t)PseudoValue_type, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoValue::typeCode(){}
    QC_PSEUDOVALUE->addMethod("typeCode", (q_method_n_t)PseudoValue_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoValue::val(){}
    QC_PSEUDOVALUE->addMethod("val", (q_method_n_t)PseudoValue_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDOVALUE;
}
