/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_Date.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/QoreHashNodeIntern.h"
/* Qore class Qore::<date> */

qore_classid_t CID_PSEUDODATE;
QoreClass* QC_PSEUDODATE;

// bool PseudoDate::absolute(){}
static QoreValue PseudoDate_absolute(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 68 "Pseudo_QC_Date.qpp"
    return dt->isAbsolute();
}

// *string PseudoDate::currentZoneName(){}
static QoreValue PseudoDate_currentZoneName(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 400 "Pseudo_QC_Date.qpp"
    if (dt->isRelative())
      return QoreValue();

   bool is_dst;
   const char *zone_name;

   AbstractQoreZoneInfo::getUTCOffset(dt->getZone(), dt->getEpochSecondsUTC(), is_dst, zone_name);
   return new QoreStringNode(zone_name);
}

// int PseudoDate::dayNumber(){}
static QoreValue PseudoDate_dayNumber(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 544 "Pseudo_QC_Date.qpp"
    return dt->getDayNumber();
}

// int PseudoDate::dayOfWeek(){}
static QoreValue PseudoDate_dayOfWeek(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 563 "Pseudo_QC_Date.qpp"
    return dt->getDayOfWeek();
}

// int PseudoDate::days(){}
static QoreValue PseudoDate_days(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 130 "Pseudo_QC_Date.qpp"
    return dt->getDay();
}

// int PseudoDate::durationMicroseconds(){}
static QoreValue PseudoDate_durationMicroseconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 312 "Pseudo_QC_Date.qpp"
    return dt->getRelativeMicroseconds();
}

// int PseudoDate::durationMilliseconds(){}
static QoreValue PseudoDate_durationMilliseconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 283 "Pseudo_QC_Date.qpp"
    return dt->getRelativeMilliseconds();
}

// int PseudoDate::durationSeconds(){}
static QoreValue PseudoDate_durationSeconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 252 "Pseudo_QC_Date.qpp"
    return dt->getRelativeSeconds();
}

// float PseudoDate::durationSecondsFloat(){}
static QoreValue PseudoDate_durationSecondsFloat(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 337 "Pseudo_QC_Date.qpp"
    return dt->getRelativeSecondsDouble();
}

// string PseudoDate::format(string format){}
static QoreValue PseudoDate_format_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* format = HARD_QORE_VALUE_STRING(args, 0);
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 356 "Pseudo_QC_Date.qpp"
    QoreStringNode *rv = new QoreStringNode;
   dt->format(*rv, format->getBuffer());
   return rv;
}

// int PseudoDate::getEpochSeconds(){}
static QoreValue PseudoDate_getEpochSeconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 497 "Pseudo_QC_Date.qpp"
    return dt->isAbsolute() ? dt->getEpochSecondsUTC() : 0;
}

// int PseudoDate::getEpochSecondsLocalTime(){}
static QoreValue PseudoDate_getEpochSecondsLocalTime(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 511 "Pseudo_QC_Date.qpp"
    return dt->isAbsolute() ? dt->getEpochSeconds() : 0;
}

// int PseudoDate::getUtcOffset(){}
static QoreValue PseudoDate_getUtcOffset(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 420 "Pseudo_QC_Date.qpp"
    return AbstractQoreZoneInfo::getUTCOffset(dt->getZone());
}

// int PseudoDate::hours(){}
static QoreValue PseudoDate_hours(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 147 "Pseudo_QC_Date.qpp"
    return dt->getHour();
}

// hash<DateTimeInfo> PseudoDate::info(){}
static QoreValue PseudoDate_info(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 527 "Pseudo_QC_Date.qpp"
    return date_info(*dt);
}

// bool PseudoDate::intp(){}
static QoreValue PseudoDate_intp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 450 "Pseudo_QC_Date.qpp"
    return true;
}

// bool PseudoDate::isDst(){}
static QoreValue PseudoDate_isDst(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 371 "Pseudo_QC_Date.qpp"
    bool is_dst;

   AbstractQoreZoneInfo::getUTCOffset(dt->getZone(), dt->getEpochSecondsUTC(), is_dst);
   return is_dst;
}

// int PseudoDate::isoDayOfWeek(){}
static QoreValue PseudoDate_isoDayOfWeek(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 582 "Pseudo_QC_Date.qpp"
    int d = dt->getDayOfWeek();
   return (!d ? 7 : d);
}

// hash<IsoWeekInfo> PseudoDate::isoWeekHash(){}
static QoreValue PseudoDate_isoWeekHash(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 603 "Pseudo_QC_Date.qpp"
    int year, week, day;
    dt->getISOWeek(year, week, day);

    QoreHashNode* h = new QoreHashNode(hashdeclIsoWeekInfo, xsink);
    qore_hash_private* ph = qore_hash_private::get(*h);
    ph->setKeyValueIntern("year", year);
    ph->setKeyValueIntern("week", week);
    ph->setKeyValueIntern("day", day);

    return h;
}

// string PseudoDate::isoWeekString(){}
static QoreValue PseudoDate_isoWeekString(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 631 "Pseudo_QC_Date.qpp"
    int year, week, day;
   dt->getISOWeek(year, week, day);
   QoreStringNode *str = new QoreStringNode();
   str->sprintf("%04d-W%02d-%d", year, week, day);
   return str;
}

// int PseudoDate::microseconds(){}
static QoreValue PseudoDate_microseconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 221 "Pseudo_QC_Date.qpp"
    return dt->getMicrosecond();
}

// date PseudoDate::midnight(){}
static QoreValue PseudoDate_midnight(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 481 "Pseudo_QC_Date.qpp"
    DateTimeNode* rv = new DateTimeNode(*dt);
   rv->setTime(0, 0, 0);
   return rv;
}

// int PseudoDate::milliseconds(){}
static QoreValue PseudoDate_milliseconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 202 "Pseudo_QC_Date.qpp"
    return dt->getMillisecond();
}

// int PseudoDate::minutes(){}
static QoreValue PseudoDate_minutes(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 164 "Pseudo_QC_Date.qpp"
    return dt->getMinute();
}

// int PseudoDate::months(){}
static QoreValue PseudoDate_months(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 113 "Pseudo_QC_Date.qpp"
    return dt->getMonth();
}

// bool PseudoDate::relative(){}
static QoreValue PseudoDate_relative(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 81 "Pseudo_QC_Date.qpp"
    return dt->isRelative();
}

// int PseudoDate::seconds(){}
static QoreValue PseudoDate_seconds(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 183 "Pseudo_QC_Date.qpp"
    return dt->getSecond();
}

// bool PseudoDate::strp(){}
static QoreValue PseudoDate_strp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 464 "Pseudo_QC_Date.qpp"
    return true;
}

// int PseudoDate::typeCode(){}
static QoreValue PseudoDate_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 55 "Pseudo_QC_Date.qpp"
    return NT_DATE;
}

// bool PseudoDate::val(){}
static QoreValue PseudoDate_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 436 "Pseudo_QC_Date.qpp"
    return dt->hasValue();
}

// int PseudoDate::years(){}
static QoreValue PseudoDate_years(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 96 "Pseudo_QC_Date.qpp"
    return dt->getYear();
}

// *TimeZone PseudoDate::zone(){}
static QoreValue PseudoDate_zone(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode *dt = v.get<DateTimeNode>();
# 387 "Pseudo_QC_Date.qpp"
    return dt->isAbsolute() ? new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(dt->getZone())) : 0;
}

DLLLOCAL void preinitPseudoDateClass() {
    QC_PSEUDODATE = new QoreClass("<date>", "::Qore::<date>", QDOM_DEFAULT);
    CID_PSEUDODATE = QC_PSEUDODATE->getID();
    QC_PSEUDODATE->setSystem();
}

DLLLOCAL QoreClass* initPseudoDateClass() {
    if (!QC_PSEUDODATE)
        preinitPseudoDateClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDODATE->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // bool PseudoDate::absolute(){}
    QC_PSEUDODATE->addMethod("absolute", (q_method_n_t)PseudoDate_absolute, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // *string PseudoDate::currentZoneName(){}
    QC_PSEUDODATE->addMethod("currentZoneName", (q_method_n_t)PseudoDate_currentZoneName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int PseudoDate::dayNumber(){}
    QC_PSEUDODATE->addMethod("dayNumber", (q_method_n_t)PseudoDate_dayNumber, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::dayOfWeek(){}
    QC_PSEUDODATE->addMethod("dayOfWeek", (q_method_n_t)PseudoDate_dayOfWeek, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::days(){}
    QC_PSEUDODATE->addMethod("days", (q_method_n_t)PseudoDate_days, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::durationMicroseconds(){}
    QC_PSEUDODATE->addMethod("durationMicroseconds", (q_method_n_t)PseudoDate_durationMicroseconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::durationMilliseconds(){}
    QC_PSEUDODATE->addMethod("durationMilliseconds", (q_method_n_t)PseudoDate_durationMilliseconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::durationSeconds(){}
    QC_PSEUDODATE->addMethod("durationSeconds", (q_method_n_t)PseudoDate_durationSeconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // float PseudoDate::durationSecondsFloat(){}
    QC_PSEUDODATE->addMethod("durationSecondsFloat", (q_method_n_t)PseudoDate_durationSecondsFloat, Public, QCF_CONSTANT, QDOM_DEFAULT, floatTypeInfo);

    // string PseudoDate::format(string format){}
    QC_PSEUDODATE->addMethod("format", (q_method_n_t)PseudoDate_format_Vs, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "format");

    // int PseudoDate::getEpochSeconds(){}
    QC_PSEUDODATE->addMethod("getEpochSeconds", (q_method_n_t)PseudoDate_getEpochSeconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::getEpochSecondsLocalTime(){}
    QC_PSEUDODATE->addMethod("getEpochSecondsLocalTime", (q_method_n_t)PseudoDate_getEpochSecondsLocalTime, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::getUtcOffset(){}
    QC_PSEUDODATE->addMethod("getUtcOffset", (q_method_n_t)PseudoDate_getUtcOffset, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::hours(){}
    QC_PSEUDODATE->addMethod("hours", (q_method_n_t)PseudoDate_hours, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash<DateTimeInfo> PseudoDate::info(){}
    QC_PSEUDODATE->addMethod("info", (q_method_n_t)PseudoDate_info, Public, QCF_CONSTANT, QDOM_DEFAULT, hashdeclDateTimeInfo->getTypeInfo(false));

    // bool PseudoDate::intp(){}
    QC_PSEUDODATE->addMethod("intp", (q_method_n_t)PseudoDate_intp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoDate::isDst(){}
    QC_PSEUDODATE->addMethod("isDst", (q_method_n_t)PseudoDate_isDst, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoDate::isoDayOfWeek(){}
    QC_PSEUDODATE->addMethod("isoDayOfWeek", (q_method_n_t)PseudoDate_isoDayOfWeek, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash<IsoWeekInfo> PseudoDate::isoWeekHash(){}
    QC_PSEUDODATE->addMethod("isoWeekHash", (q_method_n_t)PseudoDate_isoWeekHash, Public, QCF_CONSTANT, QDOM_DEFAULT, hashdeclIsoWeekInfo->getTypeInfo(false));

    // string PseudoDate::isoWeekString(){}
    QC_PSEUDODATE->addMethod("isoWeekString", (q_method_n_t)PseudoDate_isoWeekString, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoDate::microseconds(){}
    QC_PSEUDODATE->addMethod("microseconds", (q_method_n_t)PseudoDate_microseconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // date PseudoDate::midnight(){}
    QC_PSEUDODATE->addMethod("midnight", (q_method_n_t)PseudoDate_midnight, Public, QCF_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

    // int PseudoDate::milliseconds(){}
    QC_PSEUDODATE->addMethod("milliseconds", (q_method_n_t)PseudoDate_milliseconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::minutes(){}
    QC_PSEUDODATE->addMethod("minutes", (q_method_n_t)PseudoDate_minutes, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoDate::months(){}
    QC_PSEUDODATE->addMethod("months", (q_method_n_t)PseudoDate_months, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoDate::relative(){}
    QC_PSEUDODATE->addMethod("relative", (q_method_n_t)PseudoDate_relative, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoDate::seconds(){}
    QC_PSEUDODATE->addMethod("seconds", (q_method_n_t)PseudoDate_seconds, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoDate::strp(){}
    QC_PSEUDODATE->addMethod("strp", (q_method_n_t)PseudoDate_strp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoDate::typeCode(){}
    QC_PSEUDODATE->addMethod("typeCode", (q_method_n_t)PseudoDate_typeCode, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoDate::val(){}
    QC_PSEUDODATE->addMethod("val", (q_method_n_t)PseudoDate_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoDate::years(){}
    QC_PSEUDODATE->addMethod("years", (q_method_n_t)PseudoDate_years, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *TimeZone PseudoDate::zone(){}
    QC_PSEUDODATE->addMethod("zone", (q_method_n_t)PseudoDate_zone, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TIMEZONE->getOrNothingTypeInfo());

    return QC_PSEUDODATE;
}
