/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_Hash.cpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>

#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/QoreHashIterator.h"
#include "qore/intern/QoreHashListIterator.h"

extern QoreClass* QC_HASHLISTITERATOR;
extern QoreClass* QC_HASHITERATOR;
extern QoreClass* QC_HASHKEYITERATOR;
extern QoreClass* QC_HASHPAIRITERATOR;
/* Qore class Qore::<hash> */

qore_classid_t CID_PSEUDOHASH;
QoreClass* QC_PSEUDOHASH;

// bool PseudoHash::compareKeys(hash<auto> oh){}
static QoreValue PseudoHash_compareKeys_C10hash_auto_(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* oh = HARD_QORE_VALUE_HASH(args, 0);
    const QoreHashNode* h = v.get<QoreHashNode>();
# 376 "Pseudo_QC_Hash.qpp"
    if (h->size() != oh->size())
      return false;

   ConstHashIterator hi1(h);
   ConstHashIterator hi2(oh);
   while (hi1.next()) {
      hi2.next();
      if (strcmp(hi1.getKey(), hi2.getKey()))
         return false;
   }

   return true;
}

// bool PseudoHash::complexType(){}
static QoreValue PseudoHash_complexType(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 81 "Pseudo_QC_Hash.qpp"
    const qore_hash_private* hp = qore_hash_private::get(*h);
   return hp->hashdecl || hp->complexTypeInfo ? true : false;
}

// HashListIterator PseudoHash::contextIterator(){}
static QoreValue PseudoHash_contextIterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 358 "Pseudo_QC_Hash.qpp"
    return new QoreObject(QC_HASHLISTITERATOR, 0, new QoreHashListIterator(h));
}

// bool PseudoHash::empty(){}
static QoreValue PseudoHash_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 250 "Pseudo_QC_Hash.qpp"
    return h->empty();
}

// *string PseudoHash::firstKey(){}
static QoreValue PseudoHash_firstKey(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 136 "Pseudo_QC_Hash.qpp"
    const char *key = h->getFirstKey();
   return key ? new QoreStringNode(key) : 0;
}

// auto PseudoHash::firstValue(){}
static QoreValue PseudoHash_firstValue(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 154 "Pseudo_QC_Hash.qpp"
    return qore_hash_private::getFirstKeyValue(h).refSelf();
}

// bool PseudoHash::hasKey(softstring key){}
static QoreValue PseudoHash_hasKey_vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* h = v.get<QoreHashNode>();
# 208 "Pseudo_QC_Hash.qpp"
    TempEncodingHelper str(key, QCS_DEFAULT, xsink);
   if (!str)
      return false;

   return h->existsKey(str->getBuffer());
}

// bool PseudoHash::hasKeyValue(softstring key){}
static QoreValue PseudoHash_hasKeyValue_vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* h = v.get<QoreHashNode>();
# 231 "Pseudo_QC_Hash.qpp"
    TempEncodingHelper str(key, QCS_DEFAULT, xsink);
   if (!str)
      return false;

   return h->existsKeyValue(str->getBuffer());
}

// AbstractIterator PseudoHash::iterator(){}
static QoreValue PseudoHash_iterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 304 "Pseudo_QC_Hash.qpp"
    return new QoreObject(QC_HASHITERATOR, 0, new QoreHashIterator(h));
}

// HashKeyIterator PseudoHash::keyIterator(){}
static QoreValue PseudoHash_keyIterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 322 "Pseudo_QC_Hash.qpp"
    return new QoreObject(QC_HASHKEYITERATOR, 0, new QoreHashIterator(h));
}

// list<string> PseudoHash::keys(){}
static QoreValue PseudoHash_keys(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 99 "Pseudo_QC_Hash.qpp"
    return h->getKeys();
}

// *string PseudoHash::lastKey(){}
static QoreValue PseudoHash_lastKey(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 171 "Pseudo_QC_Hash.qpp"
    const char *key = h->getLastKey();
   return key ? new QoreStringNode(key) : 0;
}

// auto PseudoHash::lastValue(){}
static QoreValue PseudoHash_lastValue(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 189 "Pseudo_QC_Hash.qpp"
    return qore_hash_private::getLastKeyValue(h).refSelf();
}

// HashPairIterator PseudoHash::pairIterator(){}
static QoreValue PseudoHash_pairIterator(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 338 "Pseudo_QC_Hash.qpp"
    return new QoreObject(QC_HASHPAIRITERATOR, 0, new QoreHashIterator(h));
}

// int PseudoHash::size(){}
static QoreValue PseudoHash_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 268 "Pseudo_QC_Hash.qpp"
    return h->size();
}

// bool PseudoHash::sizep(){}
static QoreValue PseudoHash_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 398 "Pseudo_QC_Hash.qpp"
    return true;
}

// int PseudoHash::typeCode(){}
static QoreValue PseudoHash_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 64 "Pseudo_QC_Hash.qpp"
    return NT_HASH;
}

// bool PseudoHash::val(){}
static QoreValue PseudoHash_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 288 "Pseudo_QC_Hash.qpp"
    return !h->empty();
}

// list<auto> PseudoHash::values(){}
static QoreValue PseudoHash_values(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = v.get<QoreHashNode>();
# 119 "Pseudo_QC_Hash.qpp"
    return qore_hash_private::get(*h)->getValues();
}

DLLLOCAL void preinitPseudoHashClass() {
    QC_PSEUDOHASH = new QoreClass("<hash>", "::Qore::<hash>", QDOM_DEFAULT);
    CID_PSEUDOHASH = QC_PSEUDOHASH->getID();
    QC_PSEUDOHASH->setSystem();
}

DLLLOCAL QoreClass* initPseudoHashClass() {
    if (!QC_PSEUDOHASH)
        preinitPseudoHashClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOHASH->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // bool PseudoHash::compareKeys(hash<auto> oh){}
    QC_PSEUDOHASH->addMethod("compareKeys", (q_method_n_t)PseudoHash_compareKeys_C10hash_auto_, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "oh");

    // bool PseudoHash::complexType(){}
    QC_PSEUDOHASH->addMethod("complexType", (q_method_n_t)PseudoHash_complexType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // HashListIterator PseudoHash::contextIterator(){}
    QC_PSEUDOHASH->addMethod("contextIterator", (q_method_n_t)PseudoHash_contextIterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_HASHLISTITERATOR->getTypeInfo());

    // bool PseudoHash::empty(){}
    QC_PSEUDOHASH->addMethod("empty", (q_method_n_t)PseudoHash_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string PseudoHash::firstKey(){}
    QC_PSEUDOHASH->addMethod("firstKey", (q_method_n_t)PseudoHash_firstKey, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // auto PseudoHash::firstValue(){}
    QC_PSEUDOHASH->addMethod("firstValue", (q_method_n_t)PseudoHash_firstValue, Public, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo);

    // bool PseudoHash::hasKey(softstring key){}
    QC_PSEUDOHASH->addMethod("hasKey", (q_method_n_t)PseudoHash_hasKey_vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // bool PseudoHash::hasKeyValue(softstring key){}
    QC_PSEUDOHASH->addMethod("hasKeyValue", (q_method_n_t)PseudoHash_hasKeyValue_vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, softStringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // AbstractIterator PseudoHash::iterator(){}
    QC_PSEUDOHASH->addMethod("iterator", (q_method_n_t)PseudoHash_iterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTITERATOR->getTypeInfo());

    // HashKeyIterator PseudoHash::keyIterator(){}
    QC_PSEUDOHASH->addMethod("keyIterator", (q_method_n_t)PseudoHash_keyIterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_HASHKEYITERATOR->getTypeInfo());

    // list<string> PseudoHash::keys(){}
    QC_PSEUDOHASH->addMethod("keys", (q_method_n_t)PseudoHash_keys, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // *string PseudoHash::lastKey(){}
    QC_PSEUDOHASH->addMethod("lastKey", (q_method_n_t)PseudoHash_lastKey, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // auto PseudoHash::lastValue(){}
    QC_PSEUDOHASH->addMethod("lastValue", (q_method_n_t)PseudoHash_lastValue, Public, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo);

    // HashPairIterator PseudoHash::pairIterator(){}
    QC_PSEUDOHASH->addMethod("pairIterator", (q_method_n_t)PseudoHash_pairIterator, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_HASHPAIRITERATOR->getTypeInfo());

    // int PseudoHash::size(){}
    QC_PSEUDOHASH->addMethod("size", (q_method_n_t)PseudoHash_size, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoHash::sizep(){}
    QC_PSEUDOHASH->addMethod("sizep", (q_method_n_t)PseudoHash_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoHash::typeCode(){}
    QC_PSEUDOHASH->addMethod("typeCode", (q_method_n_t)PseudoHash_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoHash::val(){}
    QC_PSEUDOHASH->addMethod("val", (q_method_n_t)PseudoHash_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // list<auto> PseudoHash::values(){}
    QC_PSEUDOHASH->addMethod("values", (q_method_n_t)PseudoHash_values, Public, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo);

    return QC_PSEUDOHASH;
}
