/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractThreadResource.qpp AbstractThreadResource class (interface) definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AbstractThreadResource.h"
/* Qore class Qore::Thread::AbstractThreadResource */

qore_classid_t CID_ABSTRACTTHREADRESOURCE;
QoreClass* QC_ABSTRACTTHREADRESOURCE;

// AbstractThreadResource::constructor() {}
static void AbstractThreadResource_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 117 "QC_AbstractThreadResource.qpp"
self->setPrivate(CID_ABSTRACTTHREADRESOURCE, new AbstractQoreThreadResource(*self));
}

// AbstractThreadResource::destructor() {}
static void AbstractThreadResource_destructor(QoreObject* self, AbstractThreadResource* atr, ExceptionSink* xsink) {
# 124 "QC_AbstractThreadResource.qpp"
if (!remove_thread_resource(atr))
      atr->cleanup(xsink);
   atr->deref(xsink);
}

DLLLOCAL void preinitAbstractThreadResourceClass() {
    QC_ABSTRACTTHREADRESOURCE = new QoreClass("AbstractThreadResource", "::Qore::Thread::AbstractThreadResource", QDOM_DEFAULT);
    CID_ABSTRACTTHREADRESOURCE = QC_ABSTRACTTHREADRESOURCE->getID();
    QC_ABSTRACTTHREADRESOURCE->setSystem();
}

DLLLOCAL QoreClass* initAbstractThreadResourceClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTTHREADRESOURCE)
        preinitAbstractThreadResourceClass();

    // nothing AbstractThreadResource::cleanup();
    QC_ABSTRACTTHREADRESOURCE->addAbstractMethod("cleanup", Public, QCF_NO_FLAGS, nothingTypeInfo);

    // AbstractThreadResource::constructor() {}
    QC_ABSTRACTTHREADRESOURCE->addConstructor(AbstractThreadResource_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // AbstractThreadResource::destructor() {}
    QC_ABSTRACTTHREADRESOURCE->setDestructor((q_destructor_t)AbstractThreadResource_destructor);

    return QC_ABSTRACTTHREADRESOURCE;
}
