/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_AutoLock.qpp
 
  Qore Programming Language
 
  Copyright (C) 2003 - 2023 David Nichols
 
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AutoLock.h"
#include "qore/intern/QC_Mutex.h"
/* Qore class Qore::Thread::AutoLock */

qore_classid_t CID_AUTOLOCK;
QoreClass* QC_AUTOLOCK;

// AutoLock::constructor(Mutex mutex) {}
static void AutoLock_constructor_C5Mutex(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(mutex, SmartMutex, args, 0, CID_MUTEX, "AutoLock::constructor()", "Mutex", xsink);
    if (*xsink)
        return;
# 82 "QC_AutoLock.qpp"
QoreAutoLock *qsl = new QoreAutoLock(mutex, xsink);
   if (*xsink)
      qsl->deref(xsink);
   else
      self->setPrivate(CID_AUTOLOCK, qsl);
}

// AutoLock::copy() {}
static void AutoLock_copy(QoreObject* self, QoreObject* old, QoreAutoLock* al, ExceptionSink* xsink) {
# 107 "QC_AutoLock.qpp"
xsink->raiseException("AUTOLOCK-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoLock::destructor() {}
static void AutoLock_destructor(QoreObject* self, QoreAutoLock* al, ExceptionSink* xsink) {
# 98 "QC_AutoLock.qpp"
al->destructor(xsink);
   al->deref(xsink);
}

// nothing AutoLock::lock(){}
static QoreValue AutoLock_lock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "QC_AutoLock.qpp"
    al->lock(xsink);
    return QoreValue();
}

// int AutoLock::trylock(){}
static QoreValue AutoLock_trylock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 138 "QC_AutoLock.qpp"
    return al->trylock();
}

// nothing AutoLock::unlock(){}
static QoreValue AutoLock_unlock(QoreObject* self, QoreAutoLock* al, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 151 "QC_AutoLock.qpp"
    al->unlock(xsink);
    return QoreValue();
}

DLLLOCAL void preinitAutoLockClass() {
    QC_AUTOLOCK = new QoreClass("AutoLock", "::Qore::Thread::AutoLock", QDOM_THREAD_CLASS);
    CID_AUTOLOCK = QC_AUTOLOCK->getID();
    QC_AUTOLOCK->setSystem();
}

DLLLOCAL QoreClass* initAutoLockClass(QoreNamespace& ns) {
    if (!QC_AUTOLOCK)
        preinitAutoLockClass();

    // AutoLock::constructor(Mutex mutex) {}
    QC_AUTOLOCK->addConstructor(AutoLock_constructor_C5Mutex, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_MUTEX->getTypeInfo(), QORE_PARAM_NO_ARG, "mutex");

    // AutoLock::copy() {}
    QC_AUTOLOCK->setCopy((q_copy_t)AutoLock_copy);

    // AutoLock::destructor() {}
    QC_AUTOLOCK->setDestructor((q_destructor_t)AutoLock_destructor);

    // nothing AutoLock::lock(){}
    QC_AUTOLOCK->addMethod("lock", (q_method_n_t)AutoLock_lock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int AutoLock::trylock(){}
    QC_AUTOLOCK->addMethod("trylock", (q_method_n_t)AutoLock_trylock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing AutoLock::unlock(){}
    QC_AUTOLOCK->addMethod("unlock", (q_method_n_t)AutoLock_unlock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    return QC_AUTOLOCK;
}
