/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_BufferedStreamReader.qpp BufferedStreamReader class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/InputStream.h"
#include "qore/intern/BufferedStreamReader.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"
/* Qore class Qore::BufferedStreamReader */

qore_classid_t CID_BUFFEREDSTREAMREADER;
QoreClass* QC_BUFFEREDSTREAMREADER;

// BufferedStreamReader::constructor(Qore::InputStream is, *string encoding, int bufsize = DefaultStreamBufferSize) {}
static void BufferedStreamReader_constructor_C11InputStreamNsVi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "BufferedStreamReader::constructor()", "Qore::InputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
    int64 bufsize = HARD_QORE_VALUE_INT(args, 2);
# 69 "QC_BufferedStreamReader.qpp"
SimpleRefHolder<BufferedStreamReader> reader(new BufferedStreamReader(xsink, is, encoding ? QEM.findCreate(encoding) : QCS_DEFAULT, bufsize));
   if (*xsink)
      return;
   self->setPrivate(CID_BUFFEREDSTREAMREADER, reader.release());
   qore_object_private* o = qore_object_private::get(*self);
   const qore_class_private* cls = qore_class_private::get(*QC_BUFFEREDSTREAMREADER);
   o->setValueIntern(cls, "is", static_cast<QoreObject*>(obj_is->refSelf()), xsink);
}

DLLLOCAL void preinitBufferedStreamReaderClass() {
    QC_BUFFEREDSTREAMREADER = new QoreClass("BufferedStreamReader", "::Qore::BufferedStreamReader", QDOM_DEFAULT);
    CID_BUFFEREDSTREAMREADER = QC_BUFFEREDSTREAMREADER->getID();
    QC_BUFFEREDSTREAMREADER->setSystem();
}

DLLLOCAL QoreClass* initBufferedStreamReaderClass(QoreNamespace& ns) {
    if (!QC_BUFFEREDSTREAMREADER)
        preinitBufferedStreamReaderClass();

    // set parent class
    assert(QC_STREAMREADER);
    QC_BUFFEREDSTREAMREADER->addBuiltinVirtualBaseClass(QC_STREAMREADER);

    // private:internal members
    QC_BUFFEREDSTREAMREADER->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());

    // BufferedStreamReader::constructor(Qore::InputStream is, *string encoding, int bufsize = DefaultStreamBufferSize) {}
    QC_BUFFEREDSTREAMREADER->addConstructor(BufferedStreamReader_constructor_C11InputStreamNsVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding", bigIntTypeInfo, QoreSimpleValue().assign((int64)DefaultStreamBufferSize), "bufsize");
    ns.addConstant("DefaultStreamBufferSize", ((int64)4096));

    return QC_BUFFEREDSTREAMREADER;
}
