/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashIterator.qpp HashIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashIterator.h"
/* Qore class Qore::HashIterator */

qore_classid_t CID_HASHITERATOR;
QoreClass* QC_HASHITERATOR;

// HashIterator::constructor(hash<auto> h) {}
static void HashIterator_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 70 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator(h));
}

// HashIterator::constructor() {}
static void HashIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 81 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator);
}

// HashIterator::copy() {}
static void HashIterator_copy(QoreObject* self, QoreObject* old, QoreHashIterator* i, ExceptionSink* xsink) {
# 91 "QC_HashIterator.qpp"
self->setPrivate(CID_HASHITERATOR, new QoreHashIterator(*i));
}

// bool HashIterator::empty(){}
static QoreValue HashIterator_empty(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 146 "QC_HashIterator.qpp"
    return i->empty();
}

// bool HashIterator::first(){}
static QoreValue HashIterator_first(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 161 "QC_HashIterator.qpp"
    return i->first();
}

// string HashIterator::getKey(){}
static QoreValue HashIterator_getKey(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 256 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getKey(xsink);
}

// auto HashIterator::getKeyValue(){}
static QoreValue HashIterator_getKeyValue(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 216 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getReferencedKeyValue(xsink);
}

// auto HashIterator::getValue(){}
static QoreValue HashIterator_getValue(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 195 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getReferencedValue(xsink);
}

// hash<auto> HashIterator::getValuePair(){}
static QoreValue HashIterator_getValuePair(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 237 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getReferencedValuePair(xsink);
}

// bool HashIterator::last(){}
static QoreValue HashIterator_last(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 176 "QC_HashIterator.qpp"
    return i->last();
}

// bool HashIterator::next(){}
static QoreValue HashIterator_next(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 110 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// bool HashIterator::prev(){}
static QoreValue HashIterator_prev(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 131 "QC_HashIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing HashIterator::reset(){}
static QoreValue HashIterator_reset(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 285 "QC_HashIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool HashIterator::valid(){}
static QoreValue HashIterator_valid(QoreObject* self, QoreHashIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 271 "QC_HashIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitHashIteratorClass() {
    QC_HASHITERATOR = new QoreClass("HashIterator", "::Qore::HashIterator", QDOM_DEFAULT);
    CID_HASHITERATOR = QC_HASHITERATOR->getID();
    QC_HASHITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHITERATOR)
        preinitHashIteratorClass();

    // set parent class
    assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
    QC_HASHITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

    // HashIterator::constructor(hash<auto> h) {}
    QC_HASHITERATOR->addConstructor(HashIterator_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // HashIterator::constructor() {}
    QC_HASHITERATOR->addConstructor(HashIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HashIterator::copy() {}
    QC_HASHITERATOR->setCopy((q_copy_t)HashIterator_copy);

    // bool HashIterator::empty(){}
    QC_HASHITERATOR->addMethod("empty", (q_method_n_t)HashIterator_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashIterator::first(){}
    QC_HASHITERATOR->addMethod("first", (q_method_n_t)HashIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string HashIterator::getKey(){}
    QC_HASHITERATOR->addMethod("getKey", (q_method_n_t)HashIterator_getKey, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // auto HashIterator::getKeyValue(){}
    QC_HASHITERATOR->addMethod("getKeyValue", (q_method_n_t)HashIterator_getKeyValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // auto HashIterator::getValue(){}
    QC_HASHITERATOR->addMethod("getValue", (q_method_n_t)HashIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // hash<auto> HashIterator::getValuePair(){}
    QC_HASHITERATOR->addMethod("getValuePair", (q_method_n_t)HashIterator_getValuePair, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    // bool HashIterator::last(){}
    QC_HASHITERATOR->addMethod("last", (q_method_n_t)HashIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashIterator::next(){}
    QC_HASHITERATOR->addMethod("next", (q_method_n_t)HashIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool HashIterator::prev(){}
    QC_HASHITERATOR->addMethod("prev", (q_method_n_t)HashIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing HashIterator::reset(){}
    QC_HASHITERATOR->addMethod("reset", (q_method_n_t)HashIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool HashIterator::valid(){}
    QC_HASHITERATOR->addMethod("valid", (q_method_n_t)HashIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_HASHITERATOR;
}
