/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Mutex.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Mutex.h"
#include "qore/intern/QC_AbstractSmartLock.h"
/* Qore class Qore::Thread::Mutex */

qore_classid_t CID_MUTEX;
QoreClass* QC_MUTEX;

// Mutex::constructor() {}
static void Mutex_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 74 "QC_Mutex.qpp"
self->setPrivate(CID_MUTEX, new SmartMutex);
}

// Mutex::copy() {}
static void Mutex_copy(QoreObject* self, QoreObject* old, SmartMutex* m, ExceptionSink* xsink) {
# 99 "QC_Mutex.qpp"
self->setPrivate(CID_MUTEX, new SmartMutex);
}

// Mutex::destructor() {}
static void Mutex_destructor(QoreObject* self, SmartMutex* m, ExceptionSink* xsink) {
# 88 "QC_Mutex.qpp"
m->destructor(xsink);
    m->deref(xsink);
}

// nothing Mutex::lock(){}
static QoreValue Mutex_lock(QoreObject* self, SmartMutex* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 114 "QC_Mutex.qpp"
    m->grab(xsink);
    return QoreValue();
}

// int Mutex::lock(timeout timeout_ms){}
static QoreValue Mutex_lock_Vt(QoreObject* self, SmartMutex* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 136 "QC_Mutex.qpp"
    return m->grab(xsink, timeout_ms);
}

// int Mutex::trylock(){}
static QoreValue Mutex_trylock(QoreObject* self, SmartMutex* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 152 "QC_Mutex.qpp"
    return m->tryGrab();
}

// nothing Mutex::unlock(){}
static QoreValue Mutex_unlock(QoreObject* self, SmartMutex* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 165 "QC_Mutex.qpp"
    m->release(xsink);
    return QoreValue();
}

DLLLOCAL void preinitMutexClass() {
    QC_MUTEX = new QoreClass("Mutex", "::Qore::Thread::Mutex", QDOM_THREAD_CLASS);
    CID_MUTEX = QC_MUTEX->getID();
    QC_MUTEX->setSystem();
}

DLLLOCAL QoreClass* initMutexClass(QoreNamespace& ns) {
    if (!QC_MUTEX)
        preinitMutexClass();

    // set parent class
    assert(QC_ABSTRACTSMARTLOCK);
    QC_MUTEX->addBuiltinVirtualBaseClass(QC_ABSTRACTSMARTLOCK);

    // Mutex::constructor() {}
    QC_MUTEX->addConstructor(Mutex_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Mutex::copy() {}
    QC_MUTEX->setCopy((q_copy_t)Mutex_copy);

    // Mutex::destructor() {}
    QC_MUTEX->setDestructor((q_destructor_t)Mutex_destructor);

    // nothing Mutex::lock(){}
    QC_MUTEX->addMethod("lock", (q_method_n_t)Mutex_lock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Mutex::lock(timeout timeout_ms){}
    QC_MUTEX->addMethod("lock", (q_method_n_t)Mutex_lock_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // int Mutex::trylock(){}
    QC_MUTEX->addMethod("trylock", (q_method_n_t)Mutex_trylock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Mutex::unlock(){}
    QC_MUTEX->addMethod("unlock", (q_method_n_t)Mutex_unlock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    return QC_MUTEX;
}
