/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashReverseIterator.qpp HashReverseIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreObjectIterator.h"

extern QoreClass* QC_OBJECTITERATOR;
/* Qore class Qore::ObjectReverseIterator */

qore_classid_t CID_OBJECTREVERSEITERATOR;
QoreClass* QC_OBJECTREVERSEITERATOR;

// ObjectReverseIterator::constructor(object o) {}
static void ObjectReverseIterator_constructor_Vo(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* o = HARD_QORE_VALUE_OBJECT(args, 0);
# 86 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator(o));
}

// ObjectReverseIterator::constructor() {}
static void ObjectReverseIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 97 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator);
}

// ObjectReverseIterator::copy() {}
static void ObjectReverseIterator_copy(QoreObject* self, QoreObject* old, QoreObjectReverseIterator* i, ExceptionSink* xsink) {
# 107 "QC_ObjectReverseIterator.qpp"
self->setPrivate(CID_OBJECTREVERSEITERATOR, new QoreObjectReverseIterator(*i));
}

// bool ObjectReverseIterator::first(){}
static QoreValue ObjectReverseIterator_first(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 170 "QC_ObjectReverseIterator.qpp"
    return i->last();
}

// bool ObjectReverseIterator::last(){}
static QoreValue ObjectReverseIterator_last(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 185 "QC_ObjectReverseIterator.qpp"
    return i->first();
}

// bool ObjectReverseIterator::next(){}
static QoreValue ObjectReverseIterator_next(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 129 "QC_ObjectReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->prev();
}

// bool ObjectReverseIterator::prev(){}
static QoreValue ObjectReverseIterator_prev(QoreObject* self, QoreObjectReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 153 "QC_ObjectReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->next();
}

DLLLOCAL void preinitObjectReverseIteratorClass() {
    QC_OBJECTREVERSEITERATOR = new QoreClass("ObjectReverseIterator", "::Qore::ObjectReverseIterator", QDOM_DEFAULT);
    CID_OBJECTREVERSEITERATOR = QC_OBJECTREVERSEITERATOR->getID();
    QC_OBJECTREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initObjectReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_OBJECTREVERSEITERATOR)
        preinitObjectReverseIteratorClass();

    // set parent class
    assert(QC_OBJECTITERATOR);
    QC_OBJECTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_OBJECTITERATOR);

    // ObjectReverseIterator::constructor(object o) {}
    QC_OBJECTREVERSEITERATOR->addConstructor(ObjectReverseIterator_constructor_Vo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "o");

    // ObjectReverseIterator::constructor() {}
    QC_OBJECTREVERSEITERATOR->addConstructor(ObjectReverseIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // ObjectReverseIterator::copy() {}
    QC_OBJECTREVERSEITERATOR->setCopy((q_copy_t)ObjectReverseIterator_copy);

    // bool ObjectReverseIterator::first(){}
    QC_OBJECTREVERSEITERATOR->addMethod("first", (q_method_n_t)ObjectReverseIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ObjectReverseIterator::last(){}
    QC_OBJECTREVERSEITERATOR->addMethod("last", (q_method_n_t)ObjectReverseIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ObjectReverseIterator::next(){}
    QC_OBJECTREVERSEITERATOR->addMethod("next", (q_method_n_t)ObjectReverseIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool ObjectReverseIterator::prev(){}
    QC_OBJECTREVERSEITERATOR->addMethod("prev", (q_method_n_t)ObjectReverseIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    return QC_OBJECTREVERSEITERATOR;
}
