/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_PipeInputStream.qpp PipeInputStream class definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/StreamPipe.h"
/* Qore class Qore::PipeInputStream */

qore_classid_t CID_PIPEINPUTSTREAM;
QoreClass* QC_PIPEINPUTSTREAM;

// private PipeInputStream::constructor() {}
static void PipeInputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 56 "QC_PipeInputStream.qpp"
assert(false);
}

// nothing PipeInputStream::finishClose(){}
static QoreValue PipeInputStream_finishClose(QoreObject* self, PipeInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 86 "QC_PipeInputStream.qpp"
    is->finishClose();
    return QoreValue();
}

// int PipeInputStream::peek(){}
static QoreValue PipeInputStream_peek(QoreObject* self, PipeInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 79 "QC_PipeInputStream.qpp"
    return is->peek(xsink);
}

// *binary PipeInputStream::read(int limit){}
static QoreValue PipeInputStream_read_Vi(QoreObject* self, PipeInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 64 "QC_PipeInputStream.qpp"
    if (limit <= 0) {
        xsink->raiseException("INPUT-STREAM-ERROR", "PipeInputStream::read() called with non-positive limit %lld", limit);
        return 0;
    }
    SimpleRefHolder<BinaryNode> result(new BinaryNode);
    result->preallocate(limit);
    int64 count = is->read(const_cast<void*>(result->getPtr()), limit, xsink);
    result->setSize(count);
    return count ? result.release() : 0;
}

// nothing PipeInputStream::reportError(hash ex){}
static QoreValue PipeInputStream_reportError_Vh(QoreObject* self, PipeInputStream* is, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* ex = HARD_QORE_VALUE_HASH(args, 0);
# 94 "QC_PipeInputStream.qpp"
    is->reportError(ex);
    return QoreValue();
}

DLLLOCAL void preinitPipeInputStreamClass() {
    QC_PIPEINPUTSTREAM = new QoreClass("PipeInputStream", "::Qore::PipeInputStream", QDOM_DEFAULT);
    CID_PIPEINPUTSTREAM = QC_PIPEINPUTSTREAM->getID();
    QC_PIPEINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initPipeInputStreamClass(QoreNamespace& ns) {
    if (!QC_PIPEINPUTSTREAM)
        preinitPipeInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_PIPEINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    QC_PIPEINPUTSTREAM->setFinal();

    // private PipeInputStream::constructor() {}
    QC_PIPEINPUTSTREAM->addConstructor(PipeInputStream_constructor, Private, QCF_NO_FLAGS, QDOM_DEFAULT);

    // nothing PipeInputStream::finishClose(){}
    QC_PIPEINPUTSTREAM->addMethod("finishClose", (q_method_n_t)PipeInputStream_finishClose, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int PipeInputStream::peek(){}
    QC_PIPEINPUTSTREAM->addMethod("peek", (q_method_n_t)PipeInputStream_peek, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary PipeInputStream::read(int limit){}
    QC_PIPEINPUTSTREAM->addMethod("read", (q_method_n_t)PipeInputStream_read_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "limit");

    // nothing PipeInputStream::reportError(hash ex){}
    QC_PIPEINPUTSTREAM->addMethod("reportError", (q_method_n_t)PipeInputStream_reportError_Vh, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, hashTypeInfo, QORE_PARAM_NO_ARG, "ex");

    return QC_PIPEINPUTSTREAM;
}
