/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_PipeOutputStream.qpp PipeOutputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/StreamPipe.h"
/* Qore class Qore::PipeOutputStream */

qore_classid_t CID_PIPEOUTPUTSTREAM;
QoreClass* QC_PIPEOUTPUTSTREAM;

// nothing PipeOutputStream::close(){}
static QoreValue PipeOutputStream_close(QoreObject* self, PipeOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 65 "QC_PipeOutputStream.qpp"
    os->close(xsink);
    return QoreValue();
}

// private PipeOutputStream::constructor() {}
static void PipeOutputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 56 "QC_PipeOutputStream.qpp"
assert(false);
}

// nothing PipeOutputStream::reportError(hash ex){}
static QoreValue PipeOutputStream_reportError_Vh(QoreObject* self, PipeOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* ex = HARD_QORE_VALUE_HASH(args, 0);
# 82 "QC_PipeOutputStream.qpp"
    os->reportError(ex);
    return QoreValue();
}

// nothing PipeOutputStream::write(binary data){}
static QoreValue PipeOutputStream_write_Vy(QoreObject* self, PipeOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 74 "QC_PipeOutputStream.qpp"
    os->write(data->getPtr(), data->size(), xsink);
    return QoreValue();
}

DLLLOCAL void preinitPipeOutputStreamClass() {
    QC_PIPEOUTPUTSTREAM = new QoreClass("PipeOutputStream", "::Qore::PipeOutputStream", QDOM_DEFAULT);
    CID_PIPEOUTPUTSTREAM = QC_PIPEOUTPUTSTREAM->getID();
    QC_PIPEOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initPipeOutputStreamClass(QoreNamespace& ns) {
    if (!QC_PIPEOUTPUTSTREAM)
        preinitPipeOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_PIPEOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    QC_PIPEOUTPUTSTREAM->setFinal();

    // nothing PipeOutputStream::close(){}
    QC_PIPEOUTPUTSTREAM->addMethod("close", (q_method_n_t)PipeOutputStream_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // private PipeOutputStream::constructor() {}
    QC_PIPEOUTPUTSTREAM->addConstructor(PipeOutputStream_constructor, Private, QCF_NO_FLAGS, QDOM_DEFAULT);

    // nothing PipeOutputStream::reportError(hash ex){}
    QC_PIPEOUTPUTSTREAM->addMethod("reportError", (q_method_n_t)PipeOutputStream_reportError_Vh, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, hashTypeInfo, QORE_PARAM_NO_ARG, "ex");

    // nothing PipeOutputStream::write(binary data){}
    QC_PIPEOUTPUTSTREAM->addMethod("write", (q_method_n_t)PipeOutputStream_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_PIPEOUTPUTSTREAM;
}
