/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_RangeIterator.qpp RangeIterator class definition */
/*
    Qore Programming Language

    Copyright 2003 - 2023 Qore Technologies s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/RangeIterator.h"
/* Qore class Qore::RangeIterator */

qore_classid_t CID_RANGEITERATOR;
QoreClass* QC_RANGEITERATOR;

// RangeIterator::constructor(int start, int stop, int step = 1, auto val) {}
static void RangeIterator_constructor_ViViViVA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 stop = HARD_QORE_VALUE_INT(args, 1);
    int64 step = HARD_QORE_VALUE_INT(args, 2);
    QoreValue val = get_param_value(args, 3);
# 81 "QC_RangeIterator.qpp"
ReferenceHolder<RangeIterator> r(new RangeIterator(start, stop, step, val, xsink), xsink);
    if (*xsink)
        return;
    self->setPrivate(CID_RANGEITERATOR, r.release());
}

// RangeIterator::constructor(int stop) {}
static void RangeIterator_constructor_Vi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 stop = HARD_QORE_VALUE_INT(args, 0);
# 100 "QC_RangeIterator.qpp"
ReferenceHolder<RangeIterator> r(new RangeIterator(0, stop, 1, QoreValue(), xsink), xsink);
    if (*xsink)
        return;
    self->setPrivate(CID_RANGEITERATOR, r.release());
}

// RangeIterator::copy() {}
static void RangeIterator_copy(QoreObject* self, QoreObject* old, RangeIterator* i, ExceptionSink* xsink) {
# 113 "QC_RangeIterator.qpp"
self->setPrivate(CID_RANGEITERATOR, new RangeIterator(*i));
}

// auto RangeIterator::getValue(){}
static QoreValue RangeIterator_getValue(QoreObject* self, RangeIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 152 "QC_RangeIterator.qpp"
    if (i->check(xsink))
        return QoreValue();
    return i->getValue(xsink);
}

// bool RangeIterator::next(){}
static QoreValue RangeIterator_next(QoreObject* self, RangeIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 131 "QC_RangeIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->next();
}

// nothing RangeIterator::reset(){}
static QoreValue RangeIterator_reset(QoreObject* self, RangeIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 185 "QC_RangeIterator.qpp"
    if (i->check(xsink))
        return QoreValue();
    i->reset();
    return QoreValue();
}

// bool RangeIterator::valid(){}
static QoreValue RangeIterator_valid(QoreObject* self, RangeIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 201 "QC_RangeIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitRangeIteratorClass() {
    QC_RANGEITERATOR = new QoreClass("RangeIterator", "::Qore::RangeIterator", QDOM_DEFAULT);
    CID_RANGEITERATOR = QC_RANGEITERATOR->getID();
    QC_RANGEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initRangeIteratorClass(QoreNamespace& ns) {
    if (!QC_RANGEITERATOR)
        preinitRangeIteratorClass();

    // set parent class
    assert(QC_ABSTRACTITERATOR);
    QC_RANGEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

    // RangeIterator::constructor(int start, int stop, int step = 1, auto val) {}
    QC_RANGEITERATOR->addConstructor(RangeIterator_constructor_ViViViVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 4, bigIntTypeInfo, QORE_PARAM_NO_ARG, "start", bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop", bigIntTypeInfo, QoreSimpleValue().assign((int64)1), "step", autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // RangeIterator::constructor(int stop) {}
    QC_RANGEITERATOR->addConstructor(RangeIterator_constructor_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop");

    // RangeIterator::copy() {}
    QC_RANGEITERATOR->setCopy((q_copy_t)RangeIterator_copy);

    // auto RangeIterator::getValue(){}
    QC_RANGEITERATOR->addMethod("getValue", (q_method_n_t)RangeIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // bool RangeIterator::next(){}
    QC_RANGEITERATOR->addMethod("next", (q_method_n_t)RangeIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing RangeIterator::reset(){}
    QC_RANGEITERATOR->addMethod("reset", (q_method_n_t)RangeIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool RangeIterator::valid(){}
    QC_RANGEITERATOR->addMethod("valid", (q_method_n_t)RangeIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_RANGEITERATOR;
}
# 205 "QC_RangeIterator.qpp"
# 261 "QC_RangeIterator.qpp"
// RangeIterator xrange(int start, int stop, int step = 1, auto val) {}
static QoreValue f_xrange_ViViViVA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 stop = HARD_QORE_VALUE_INT(args, 1);
    int64 step = HARD_QORE_VALUE_INT(args, 2);
    QoreValue val = get_param_value(args, 3);
# 253 "QC_RangeIterator.qpp"
    ReferenceHolder<RangeIterator> r(new RangeIterator(start, stop, step, val, xsink), xsink);
    if (*xsink)
        return QoreValue();

    return new QoreObject(QC_RANGEITERATOR, 0, r.release());
}

// RangeIterator xrange(int stop) {}
static QoreValue f_xrange_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 stop = HARD_QORE_VALUE_INT(args, 0);
# 283 "QC_RangeIterator.qpp"
    ReferenceHolder<RangeIterator> r(new RangeIterator(0, stop, 1, QoreValue(), xsink), xsink);
    if (*xsink)
        return QoreValue();

    return new QoreObject(QC_RANGEITERATOR, 0, r.release());
}


DLLLOCAL void init_RangeIterator_functions(QoreNamespace& ns) {
    // RangeIterator xrange(int start, int stop, int step = 1, auto val) {}
    ns.addBuiltinVariant("xrange", (q_func_n_t)f_xrange_ViViViVA, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_RANGEITERATOR->getTypeInfo(), 4, bigIntTypeInfo, QORE_PARAM_NO_ARG, "start", bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop", bigIntTypeInfo, QoreSimpleValue().assign((int64)1), "step", autoTypeInfo, QORE_PARAM_NO_ARG, "val");
    // RangeIterator xrange(int stop) {}
    ns.addBuiltinVariant("xrange", (q_func_n_t)f_xrange_Vi, QCF_CONSTANT, QDOM_DEFAULT, QC_RANGEITERATOR->getTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "stop");

}
