/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    qc_option.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>

#include <openssl/evp.h>

#include <atomic>

#define QORE_CONST_HAVE_ATOMIC_MACROS std::atomic_int{}.is_lock_free()

#ifdef HAVE_CHECK_STACK_POS
#define QORE_CONST_HAVE_CHECK_STACK_POS 1
#else
#define QORE_CONST_HAVE_CHECK_STACK_POS 0
#endif

#ifdef HAVE_SIGNAL_HANDLING
#define QORE_CONST_HAVE_SIGNAL_HANDLING 1
#else
#define QORE_CONST_HAVE_SIGNAL_HANDLING 0
#endif

#ifdef DEBUG
#define QORE_CONST_DEBUG 1
#else
#define QORE_CONST_DEBUG 0
#endif

#ifdef HAVE_ROUND
#define QORE_CONST_HAVE_ROUND 1
#else
#define QORE_CONST_HAVE_ROUND 0
#endif

#ifdef HAVE_SETEUID
#define QORE_CONST_HAVE_SETEUID 1
#else
#define QORE_CONST_HAVE_SETEUID 0
#endif

#ifdef HAVE_SETEGID
#define QORE_CONST_HAVE_SETEGID 1
#else
#define QORE_CONST_HAVE_SETEGID 0
#endif

#ifdef HAVE_SYSTEM
#define QORE_CONST_HAVE_SYSTEM 1
#else
#define QORE_CONST_HAVE_SYSTEM 0
#endif

#ifdef HAVE_KILL
#define QORE_CONST_HAVE_KILL 1
#else
#define QORE_CONST_HAVE_KILL 0
#endif

#ifdef HAVE_FORK
#define QORE_CONST_HAVE_FORK 1
#else
#define QORE_CONST_HAVE_FORK 0
#endif

#ifdef HAVE_GETPPID
#define QORE_CONST_HAVE_GETPPID 1
#else
#define QORE_CONST_HAVE_GETPPID 0
#endif

#ifdef Q_HAVE_STATVFS
#define QORE_CONST_Q_HAVE_STATVFS 1
#else
#define QORE_CONST_Q_HAVE_STATVFS 0
#endif

#ifdef HAVE_SETSID
#define QORE_CONST_HAVE_SETSID 1
#else
#define QORE_CONST_HAVE_SETSID 0
#endif

#ifdef HAVE_PWD_H
#define QORE_CONST_HAVE_PWD_H 1
#else
#define QORE_CONST_HAVE_PWD_H 0
#endif

#ifdef HAVE_TERMIOS_H
#define QORE_CONST_HAVE_TERMIOS_H 1
#else
#define QORE_CONST_HAVE_TERMIOS_H 0
#endif

#ifdef HAVE_GETUID
#define QORE_CONST_HAVE_GETUID 1
#else
#define QORE_CONST_HAVE_GETUID 0
#endif

#ifdef HAVE_CHOWN
#define QORE_CONST_HAVE_CHOWN 1
#else
#define QORE_CONST_HAVE_CHOWN 0
#endif

#ifdef HAVE_STRUCT_FLOCK
#define QORE_CONST_HAVE_STRUCT_FLOCK 1
#else
#define QORE_CONST_HAVE_STRUCT_FLOCK 0
#endif

#ifdef HAVE_OPENSSL_SHA
#define QORE_CONST_HAVE_SHA 1
#else
#define QORE_CONST_HAVE_SHA 0
#endif

#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
#define QORE_CONST_HAVE_SHA256 1
#else
#define QORE_CONST_HAVE_SHA256 0
#endif

#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
#define QORE_CONST_HAVE_SHA512 1
#else
#define QORE_CONST_HAVE_SHA512 0
#endif

#ifdef OPENSSL_HAVE_MDC2
#define QORE_CONST_HAVE_MDC2 1
#else
#define QORE_CONST_HAVE_MDC2 0
#endif

#ifndef OPENSSL_NO_MD2
#define QORE_CONST_HAVE_MD2 1
#else
#define QORE_CONST_HAVE_MD2 0
#endif

#ifndef OPENSSL_NO_RC5
#define QORE_CONST_HAVE_RC5 1
#else
#define QORE_CONST_HAVE_RC5 0
#endif

#ifndef HAVE_OPENSSL_INIT_CRYPTO
#define QORE_CONST_HAVE_DSS 1
#else
#define QORE_CONST_HAVE_DSS 0
#endif

#ifdef HAVE_SYMLINK
#define QORE_CONST_HAVE_SYMLINK 1
#else
#define QORE_CONST_HAVE_SYMLINK 0
#endif

#if defined(HAVE_UNISTD_H) && defined(_SC_OPEN_MAX)
#define QORE_CONST_HAVE_CLOSE_ALL_FD 1
#else
#define QORE_CONST_HAVE_CLOSE_ALL_FD 0
#endif

#ifdef HAVE_GETIFADDRS
#define QORE_CONST_HAVE_GET_NETIF_LIST 1
#else
#define QORE_CONST_HAVE_GET_NETIF_LIST 0
#endif

#ifdef QORE_MANAGE_STACK
#define QORE_CONST_HAVE_MANAGE_STACK 1
#else
#define QORE_CONST_HAVE_MANAGE_STACK 0
#endif

#ifdef QORE_HAVE_GET_STACK_SIZE
#define QORE_CONST_HAVE_GET_STACK_SIZE 1
#else
#define QORE_CONST_HAVE_GET_STACK_SIZE 0
#endif

#ifdef QORE_HAVE_THREAD_NAME
#define QORE_CONST_HAVE_THREAD_NAME 1
#else
#define QORE_CONST_HAVE_THREAD_NAME 0
#endif

DLLLOCAL void init_option_constants(QoreNamespace& ns) {
# 221 "qc_option.qpp"
# 231 "qc_option.qpp"
# 234 "qc_option.qpp"
# 243 "qc_option.qpp"
# 246 "qc_option.qpp"
# 251 "qc_option.qpp"
# 254 "qc_option.qpp"
# 259 "qc_option.qpp"
# 264 "qc_option.qpp"
# 267 "qc_option.qpp"
# 272 "qc_option.qpp"
# 277 "qc_option.qpp"
# 282 "qc_option.qpp"
# 285 "qc_option.qpp"
# 290 "qc_option.qpp"
# 295 "qc_option.qpp"
# 302 "qc_option.qpp"
# 307 "qc_option.qpp"
# 312 "qc_option.qpp"
# 317 "qc_option.qpp"
# 322 "qc_option.qpp"
# 325 "qc_option.qpp"
# 328 "qc_option.qpp"
# 331 "qc_option.qpp"
# 334 "qc_option.qpp"
# 337 "qc_option.qpp"
# 340 "qc_option.qpp"
# 343 "qc_option.qpp"
# 346 "qc_option.qpp"
# 349 "qc_option.qpp"
# 352 "qc_option.qpp"
# 355 "qc_option.qpp"
# 360 "qc_option.qpp"
# 365 "qc_option.qpp"
    ns.addConstant("HAVE_ATOMIC_OPERATIONS", ((bool)QORE_CONST_HAVE_ATOMIC_MACROS));
    ns.addConstant("HAVE_CLOSE_ALL_FD", ((bool)QORE_CONST_HAVE_CLOSE_ALL_FD));
    ns.addConstant("HAVE_DETERMINISTIC_GC", ((bool)1));
    ns.addConstant("HAVE_DSS", ((bool)QORE_CONST_HAVE_DSS));
    ns.addConstant("HAVE_FILE_LOCKING", ((bool)QORE_CONST_HAVE_STRUCT_FLOCK));
    ns.addConstant("HAVE_FORK", ((bool)QORE_CONST_HAVE_FORK));
    ns.addConstant("HAVE_GETPPID", ((bool)QORE_CONST_HAVE_GETPPID));
    ns.addConstant("HAVE_GET_NETIF_LIST", ((bool)QORE_CONST_HAVE_GET_NETIF_LIST));
    ns.addConstant("HAVE_GET_STACK_SIZE", ((bool)QORE_CONST_HAVE_GET_STACK_SIZE));
    ns.addConstant("HAVE_IS_EXECUTABLE", ((bool)QORE_CONST_HAVE_PWD_H));
    ns.addConstant("HAVE_KILL", ((bool)QORE_CONST_HAVE_KILL));
    ns.addConstant("HAVE_LIBRARY_DEBUGGING", ((bool)QORE_CONST_DEBUG));
    ns.addConstant("HAVE_MANAGE_STACK", ((bool)QORE_CONST_HAVE_MANAGE_STACK));
    ns.addConstant("HAVE_MD2", ((bool)QORE_CONST_HAVE_MD2));
    ns.addConstant("HAVE_MDC2", ((bool)QORE_CONST_HAVE_MDC2));
    ns.addConstant("HAVE_RC5", ((bool)QORE_CONST_HAVE_RC5));
    ns.addConstant("HAVE_ROUND", ((bool)QORE_CONST_HAVE_ROUND));
    ns.addConstant("HAVE_RUNTIME_THREAD_STACK_TRACE", ((bool)true));
    ns.addConstant("HAVE_SETEGID", ((bool)QORE_CONST_HAVE_SETEGID));
    ns.addConstant("HAVE_SETEUID", ((bool)QORE_CONST_HAVE_SETEUID));
    ns.addConstant("HAVE_SETSID", ((bool)QORE_CONST_HAVE_SETSID));
    ns.addConstant("HAVE_SHA", ((bool)QORE_CONST_HAVE_SHA));
    ns.addConstant("HAVE_SHA224", ((bool)QORE_CONST_HAVE_SHA256));
    ns.addConstant("HAVE_SHA256", ((bool)QORE_CONST_HAVE_SHA256));
    ns.addConstant("HAVE_SHA384", ((bool)QORE_CONST_HAVE_SHA512));
    ns.addConstant("HAVE_SHA512", ((bool)QORE_CONST_HAVE_SHA512));
    ns.addConstant("HAVE_SIGNAL_HANDLING", ((bool)(QORE_CONST_HAVE_SIGNAL_HANDLING && !(qore_library_options & QLO_DISABLE_SIGNAL_HANDLING))));
    ns.addConstant("HAVE_STACK_GUARD", ((bool)QORE_CONST_HAVE_CHECK_STACK_POS));
    ns.addConstant("HAVE_STATVFS", ((bool)QORE_CONST_Q_HAVE_STATVFS));
    ns.addConstant("HAVE_SYMLINK", ((bool)QORE_CONST_HAVE_SYMLINK));
    ns.addConstant("HAVE_SYSTEM", ((bool)QORE_CONST_HAVE_SYSTEM));
    ns.addConstant("HAVE_TERMIOS", ((bool)QORE_CONST_HAVE_TERMIOS_H));
    ns.addConstant("HAVE_THREAD_NAME", ((bool)QORE_CONST_HAVE_THREAD_NAME));
    ns.addConstant("HAVE_UNIX_FILEMGT", ((bool)QORE_CONST_HAVE_CHOWN));
    ns.addConstant("HAVE_UNIX_USERMGT", ((bool)QORE_CONST_HAVE_GETUID));

}
