/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ClassConstant.cpp ClassConstant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_ClassConstant.h"
#include "QC_Class.h"
/* Qore class Qore::Reflection::ClassConstant */

qore_classid_t CID_CLASSCONSTANT;
QoreClass* QC_CLASSCONSTANT;

// ClassConstant::constructor() {}
static void ClassConstant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 46 "QC_ClassConstant.qpp"
xsink->raiseException("CLASSCONSTANT-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// Class ClassConstant::getClass(){}
static QoreValue ClassConstant_getClass(QoreObject* self, QoreReflectionClassConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 58 "QC_ClassConstant.qpp"
    return new QoreObject(QC_CLASS, c->pgm, new QoreReflectionClass(c->pgm, c->cls));
}

DLLLOCAL void preinitClassConstantClass() {
    QC_CLASSCONSTANT = new QoreClass("ClassConstant", "::Qore::Reflection::ClassConstant", QDOM_REFLECTION);
    CID_CLASSCONSTANT = QC_CLASSCONSTANT->getID();
    QC_CLASSCONSTANT->setSystem();
}

DLLLOCAL QoreClass* initClassConstantClass(QoreNamespace& ns) {
    if (!QC_CLASSCONSTANT)
        preinitClassConstantClass();

    // set parent class
    assert(QC_ABSTRACTCONSTANT);
    QC_CLASSCONSTANT->addBuiltinVirtualBaseClass(QC_ABSTRACTCONSTANT);

    // ClassConstant::constructor() {}
    QC_CLASSCONSTANT->addConstructor(ClassConstant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Class ClassConstant::getClass(){}
    QC_CLASSCONSTANT->addMethod("getClass", (q_method_n_t)ClassConstant_getClass, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_CLASS->getTypeInfo());

    return QC_CLASSCONSTANT;
}
